"use strict";
/*
Copyright 2022 The Dapr Authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ActorClient_1 = __importDefault(require("./ActorClient/ActorClient"));
class ActorProxyBuilder {
    constructor(actorTypeClass, ...args) {
        this.actorTypeClass = actorTypeClass;
        if (args.length == 1) {
            const [daprClient] = args;
            this.actorClient = new ActorClient_1.default(daprClient.getDaprHost(), daprClient.getDaprPort(), daprClient.getCommunicationProtocol(), daprClient.getOptions());
        }
        else {
            const [host, port, communicationProtocol, clientOptions] = args;
            this.actorClient = new ActorClient_1.default(host, port, communicationProtocol, clientOptions);
        }
    }
    build(actorId) {
        const actorTypeClassName = this.actorTypeClass.name;
        const actorClient = this.actorClient;
        const handler = {
            get(_target, propKey, _receiver) {
                return function (...args) {
                    return __awaiter(this, void 0, void 0, function* () {
                        const body = args.length > 0 ? args : null;
                        const res = yield actorClient.actor.invoke(actorTypeClassName, actorId, propKey, body);
                        return res;
                    });
                };
            },
        };
        // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Proxy/Proxy
        // we implement a handler that will take a method and forward it to the actor client
        const proxy = new Proxy(this.actorTypeClass, handler);
        // Return a NOT strongly typed API
        // @todo: this should return a strongly typed API as well, but requires reflection. How to do this in typescript?
        return proxy;
    }
}
exports.default = ActorProxyBuilder;
