/**
 * Contains the actor reminder data
 */
export default class ActorReminderData {
    readonly reminderName: string;
    readonly state: string | object | undefined;
    readonly dueTime: number;
    readonly ttl: number | undefined;
    readonly period: number;
    /**
     * @param reminderName the name of the actor reminder
     * @param state the state data passed to receiveReminder callback
     * @param dueTime the amount of time to delay before invoking the reminder for the first time
     * @param ttl time to duration after which the reminder will be expired and deleted
     * @param period the time interval between reminder invocations after the first invocation
     */
    constructor(reminderName: string, dueTime: number, period: number, ttl?: number, state?: string | object);
    getReminderName(): string;
    getState(): string | object | undefined;
    getDueTime(): number;
    getTtl(): number | undefined;
    getPeriod(): number;
    /**
     * Return this class as an object
     */
    toObject(): object;
    static fromObject(reminderName: string, obj: any): ActorReminderData;
}
