"use strict";
/*
Copyright 2022 The Dapr Authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const StateChangeKind_1 = __importDefault(require("./StateChangeKind"));
class StateProvider {
    constructor(actorClient) {
        this.actorClient = actorClient;
    }
    containsState(actorType, actorId, stateName) {
        return __awaiter(this, void 0, void 0, function* () {
            const rawStateValue = yield this.actorClient.actor.stateGet(actorType, actorId, stateName);
            return !!rawStateValue && rawStateValue.length > 0;
        });
    }
    // SEE https://github.com/dapr/python-sdk/blob/0f0b6f6a1cf45d2ac0c519b48fc868898d81124e/dapr/actor/runtime/_state_provider.py#L24
    tryLoadState(actorType, actorId, stateName) {
        return __awaiter(this, void 0, void 0, function* () {
            const rawStateValue = yield this.actorClient.actor.stateGet(actorType, actorId, stateName);
            if (!rawStateValue || rawStateValue.length === 0) {
                return [false, undefined];
            }
            // const result = this.serializer.deserialize(rawStateValue);
            return [true, rawStateValue];
        });
    }
    /**
     * Create and save the state through a transactional update request body. E.g.
     * [
     *     {
     *         "operation": "upsert",
     *         "request": {
     *             "key": "key1",
     *             "value": "myData"
     *         }
     *     },
     *     {
     *         "operation": "delete",
     *         "request": {
     *             "key": "key2"
     *         }
     *     }
     * ]
     * @param actorType
     * @param actorId
     * @param stateChanges
     */
    saveState(actorType, actorId, stateChanges) {
        return __awaiter(this, void 0, void 0, function* () {
            const jsonOutput = [];
            for (const state of stateChanges) {
                let operation;
                switch (state.getChangeKind()) {
                    case StateChangeKind_1.default.ADD:
                        // dapr doesn't know add, we use upsert
                        // https://github.com/dapr/dapr/blob/master/pkg/actors/transactional_state_request.go
                        operation = "upsert";
                        break;
                    case StateChangeKind_1.default.REMOVE:
                        operation = "remove";
                        break;
                    case StateChangeKind_1.default.UPDATE:
                        // dapr doesn't know add, we use upsert
                        // https://github.com/dapr/dapr/blob/master/pkg/actors/transactional_state_request.go
                        operation = "upsert";
                        break;
                    default:
                        // skip
                        continue;
                }
                jsonOutput.push({
                    operation,
                    request: {
                        key: state.getStateName(),
                        value: state.getValue(),
                    },
                });
            }
            yield this.actorClient.actor.stateTransaction(actorType, actorId, jsonOutput);
        });
    }
}
exports.default = StateProvider;
