"use strict";
/*
Copyright 2022 The Dapr Authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const UnlockResponse_1 = require("../../../types/lock/UnlockResponse");
const dapr_pb_1 = require("../../../proto/dapr/proto/runtime/v1/dapr_pb");
class GRPCClientLock {
    constructor(client) {
        this.client = client;
    }
    tryLock(storeName, resourceId, lockOwner, expiryInSeconds) {
        return __awaiter(this, void 0, void 0, function* () {
            const request = new dapr_pb_1.TryLockRequest()
                .setStoreName(storeName)
                .setResourceId(resourceId)
                .setLockOwner(lockOwner)
                .setExpiryinseconds(expiryInSeconds);
            const client = yield this.client.getClient();
            return new Promise((resolve, reject) => {
                client.tryLockAlpha1(request, (err, res) => {
                    if (err) {
                        return reject(err);
                    }
                    const wrapped = {
                        success: res.getSuccess(),
                    };
                    return resolve(wrapped);
                });
            });
        });
    }
    unlock(storeName, resourceId, lockOwner) {
        return __awaiter(this, void 0, void 0, function* () {
            const request = new dapr_pb_1.UnlockRequest().setStoreName(storeName).setResourceId(resourceId).setLockOwner(lockOwner);
            const client = yield this.client.getClient();
            return new Promise((resolve, reject) => {
                client.unlockAlpha1(request, (err, res) => {
                    if (err) {
                        return reject(err);
                    }
                    const wrapped = {
                        status: this.getUnlockResponse(res),
                    };
                    return resolve(wrapped);
                });
            });
        });
    }
    getUnlockResponse(res) {
        switch (res.getStatus()) {
            case dapr_pb_1.UnlockResponse.Status.SUCCESS:
                return UnlockResponse_1.LockStatus.Success;
            case dapr_pb_1.UnlockResponse.Status.LOCK_DOES_NOT_EXIST:
                return UnlockResponse_1.LockStatus.LockDoesNotExist;
            case dapr_pb_1.UnlockResponse.Status.LOCK_BELONGS_TO_OTHERS:
                return UnlockResponse_1.LockStatus.LockBelongsToOthers;
            default:
                return UnlockResponse_1.LockStatus.InternalError;
        }
    }
}
exports.default = GRPCClientLock;
