"use strict";
/*
Copyright 2022 The Dapr Authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const dapr_pb_1 = require("../../../proto/dapr/proto/runtime/v1/dapr_pb");
const Logger_1 = require("../../../logger/Logger");
const SerializerUtil = __importStar(require("../../../utils/Serializer.util"));
const Client_util_1 = require("../../../utils/Client.util");
// https://docs.dapr.io/reference/api/pubsub_api/
class GRPCClientPubSub {
    constructor(client) {
        this.client = client;
        this.logger = new Logger_1.Logger("GRPCClient", "PubSub", client.getOptions().logger);
    }
    // @todo: should return a specific typed Promise<TypePubSubPublishResponse> instead of Promise<any>
    publish(pubSubName, topic, data = {}, metadata) {
        return __awaiter(this, void 0, void 0, function* () {
            const msgService = new dapr_pb_1.PublishEventRequest();
            msgService.setPubsubName(pubSubName);
            msgService.setTopic(topic);
            msgService.setData(SerializerUtil.serializeGrpc(data).serializedData);
            const client = yield this.client.getClient();
            const grpcMetadata = (0, Client_util_1.createGRPCMetadata)(metadata);
            return new Promise((resolve, reject) => {
                client.publishEvent(msgService, grpcMetadata, (err, _res) => {
                    if (err) {
                        this.logger.error(`publish failed: ${err}`);
                        return reject(false);
                    }
                    // https://docs.dapr.io/reference/api/pubsub_api/#expected-http-response
                    return resolve(true);
                });
            });
        });
    }
}
exports.default = GRPCClientPubSub;
