"use strict";
/*
Copyright 2022 The Dapr Authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const HttpMethod_enum_1 = require("../../../enum/HttpMethod.enum");
// https://docs.dapr.io/reference/api/service_invocation_api/
class HTTPClientInvoker {
    constructor(client) {
        this.client = client;
    }
    invoke(appId, methodName, method = HttpMethod_enum_1.HttpMethod.GET, data, options = {}) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const headers = (_a = options.headers) !== null && _a !== void 0 ? _a : {};
            const fetchOptions = {
                method,
                headers,
            };
            if (method !== HttpMethod_enum_1.HttpMethod.GET) {
                //@ts-ignore
                fetchOptions.headers["Content-Type"] = "application/json";
            }
            if (method !== HttpMethod_enum_1.HttpMethod.GET && data !== undefined) {
                //@ts-ignore
                fetchOptions.body = JSON.stringify(data);
            }
            const result = yield this.client.execute(`/invoke/${appId}/method/${methodName}`, fetchOptions);
            return result;
        });
    }
}
exports.default = HTTPClientInvoker;
