"use strict";
/*
Copyright 2022 The Dapr Authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createConfigurationType = exports.createHTTPMetadataQueryParam = exports.createGRPCMetadata = void 0;
const grpc = __importStar(require("@grpc/grpc-js"));
/**
 * Converts a KeyValueType to a grpc.Metadata object.
 * @param metadata key value pair of metadata
 * @returns grpc.Metadata object
 */
function createGRPCMetadata(metadata = {}) {
    const grpcMetadata = new grpc.Metadata();
    for (const [key, value] of Object.entries(metadata)) {
        grpcMetadata.set(key, value);
    }
    return grpcMetadata;
}
exports.createGRPCMetadata = createGRPCMetadata;
/**
 * Converts a KeyValueType to a HTTP query parameters.
 * The query parameters are separated by "&", and the key value pair is separated by "=".
 * Each metadata key is prefixed with "metadata.".
 *
 * Example, if the metadata is { "key1": "value1", "key2": "value2" }, the query parameter will be:
 * "metadata.key1=value1&metadata.key2=value2"
 *
 * Note, the returned value does not contain the "?" prefix.
 *
 * @param metadata key value pair of metadata
 * @returns HTTP query parameter string
 */
function createHTTPMetadataQueryParam(metadata = {}) {
    let queryParam = "";
    for (const [key, value] of Object.entries(metadata)) {
        queryParam += "&" + "metadata." + encodeURIComponent(key) + "=" + encodeURIComponent(value);
    }
    // strip the first "&" if it exists
    queryParam = queryParam.substring(1);
    return queryParam;
}
exports.createHTTPMetadataQueryParam = createHTTPMetadataQueryParam;
/**
 * Converts a Map<string, common_pb.ConfigurationItemProto> to a ConfigurationType object.
 * @param Map<string, common_pb.ConfigurationItemProto>
 * @returns ConfigurationType object
 */
function createConfigurationType(configDict) {
    const configMap = {};
    configDict.forEach(function (v, k) {
        const item = {
            key: k,
            value: v.getValue(),
            version: v.getVersion(),
            metadata: v
                .getMetadataMap()
                .toObject()
                .reduce((result, [key, value]) => {
                // @ts-ignore
                result[key] = value;
                return result;
            }, {}),
        };
        configMap[k] = item;
    });
    return configMap;
}
exports.createConfigurationType = createConfigurationType;
